SystemOrganization addCategory: #'PersistentUnionFind-Tests'!
SystemOrganization addCategory: #PersistentUnionFind!

Object subclass: #PersistentArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!PersistentArray commentStamp: 'fbs 12/28/2011 11:09' prior: 0!
A PersistentArray is an array-like structure that permits one fast access to the _most recent version_ of the array, but also (slower) access to _previous versions_ of the array.!

PersistentArray subclass: #Arr
	instanceVariableNames: 'array'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

Arr class
	instanceVariableNames: 'array'!

!Arr commentStamp: 'fbs 12/28/2011 11:12' prior: 0!
An Arr is a wrapper for an array - really, a SequenceableCollection.!

Arr class
	instanceVariableNames: 'array'!

!Arr class methodsFor: 'instance creation' stamp: 'fbs 12/28/2011 11:45'!
wrapping: aSequenceableCollection
	^ self new wrapping: aSequenceableCollection.! !

!Arr methodsFor: 'accessing' stamp: 'fbs 12/28/2011 12:53'!
array
	^ array! !

!Arr methodsFor: 'converting' stamp: 'fbs 12/28/2011 12:38'!
asArray
	^ array copy.! !

!Arr methodsFor: 'testing' stamp: 'fbs 12/28/2011 11:28'!
isRef
	^ true! !

!Arr methodsFor: 'printing' stamp: 'fbs 12/28/2011 15:59'!
printOn: aStream
	aStream
		nextPutAll: 'Arr(';
		nextPutAll: array printString;
		nextPutAll: ')'.! !

!Arr methodsFor: 'private' stamp: 'fbs 12/28/2011 13:11'!
privAt: anInteger
	^ self array at: anInteger.! !

!Arr methodsFor: 'private' stamp: 'fbs 12/28/2011 13:13'!
privAt: index put: anObject
	| newDiff old |
	old := array at: index.
	array at: index put: anObject.
	newDiff := Diff index: index value: anObject in: self.
	
	^ newDiff.! !

!Arr methodsFor: 'updating' stamp: 'fbs 12/28/2011 12:26'!
reroot
	^ self.! !

!Arr methodsFor: 'private' stamp: 'fbs 12/28/2011 11:59'!
wrapping: aSequenceableCollection
	array := aSequenceableCollection.! !

PersistentArray subclass: #Diff
	instanceVariableNames: 'index value baseArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!Diff commentStamp: 'fbs 12/28/2011 11:12' prior: 0!
A Diff is a difference record. An instance is always relative to some Arr a, and means "like a, only with this element replaced with that value."!

!Diff class methodsFor: 'instance creation' stamp: 'fbs 12/28/2011 11:39'!
index: anInteger value: anObject in: aPersistentArray
	^ self new index: anInteger value: anObject in: aPersistentArray.! !

!Diff methodsFor: 'converting' stamp: 'fbs 12/28/2011 12:45'!
asArray
	| arr |
	arr := baseArray asArray.
	arr at: index put: value.
	^ arr.! !

!Diff methodsFor: 'accessing' stamp: 'fbs 12/28/2011 12:34'!
index
	^ index.! !

!Diff methodsFor: 'private' stamp: 'fbs 12/28/2011 11:40'!
index: anInteger value: anObject in: aPersistentArray
	index := anInteger.
	value := anObject.
	baseArray := aPersistentArray.! !

!Diff methodsFor: 'testing' stamp: 'fbs 12/28/2011 11:27'!
isDiff
	^ true.! !

!Diff methodsFor: 'printing' stamp: 'fbs 12/28/2011 15:59'!
printOn: aStream
	aStream
		nextPutAll: 'Diff(';
		nextPutAll: ({index printString. value printString. baseArray printString.} join: ', ');
		nextPutAll: ')'.
	! !

!Diff methodsFor: 'private' stamp: 'fbs 12/28/2011 13:11'!
privAt: anInteger
	Error signal: 'Cannot #at: a Diff'.! !

!Diff methodsFor: 'private' stamp: 'fbs 12/28/2011 13:09'!
privAt: index put: anObject! !

!Diff methodsFor: 'updating' stamp: 'fbs 12/28/2011 16:34'!
reroot
	"Shuffle the pointers of the entire structure around. THIS MEANS THAT SELF MAY TYPE!!"
	| a tNew |
	tNew := baseArray reroot.
	tNew isDiff
		ifTrue: [Error signal: 'Cannot reroot the Diff of a Diff'].
	tNew isInvalid
		ifTrue: [Error signal: 'Cannot reroot an InvalidPersistentArray'].
		
	a := tNew array.
	a at: index put: value.
	self become: tNew.
	tNew become: InvalidPersistentArray new.
	^ self.! !

!Diff methodsFor: 'accessing' stamp: 'fbs 12/28/2011 12:34'!
value
	^ value.! !

PersistentArray subclass: #InvalidPersistentArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!InvalidPersistentArray methodsFor: 'testing' stamp: 'fbs 12/28/2011 16:31'!
isInvalid
	^ true.! !

!InvalidPersistentArray methodsFor: 'updating' stamp: 'fbs 12/28/2011 16:30'!
reroot
	Error signal: 'Cannot reroot an InvalidPersistentArray.'! !

!PersistentArray class methodsFor: 'instance creation' stamp: 'fbs 12/28/2011 11:28'!
wrapping: aSequenceableCollection
	^ Arr wrapping: aSequenceableCollection! !

!PersistentArray methodsFor: 'converting' stamp: 'fbs 12/28/2011 12:46'!
asArray
	self subclassResponsibility.! !

!PersistentArray methodsFor: 'accessing' stamp: 'fbs 12/28/2011 13:12'!
at: anInteger
	^ self reroot privAt: anInteger.! !

!PersistentArray methodsFor: 'accessing' stamp: 'fbs 12/28/2011 13:14'!
at: anInteger put: anObject
	self reroot privAt: anInteger put: anObject.! !

!PersistentArray methodsFor: 'testing' stamp: 'fbs 12/28/2011 11:27'!
isDiff
	^ false.! !

!PersistentArray methodsFor: 'testing' stamp: 'fbs 12/28/2011 16:31'!
isInvalid
	^ false.! !

!PersistentArray methodsFor: 'testing' stamp: 'fbs 12/28/2011 11:27'!
isRef
	^ false.! !

!PersistentArray methodsFor: 'updating' stamp: 'fbs 12/28/2011 12:02'!
reroot
	self subclassResponsibility.! !

Object subclass: #PersistentUnionFind
	instanceVariableNames: 'ranks parents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind'!

!PersistentUnionFind commentStamp: 'fbs 12/28/2011 11:07' prior: 0!
A union-find data structure maintains a _partition_ of objects in some structure. That is, it implicitly defines an equivalence relation between the nodes of some structure.

The structure provides two methods - _find_ returns the representative of some class of nodes, and _union_ merges two classes of nodes.

* ranks is an array of the ranks (sizes) of the various equivalence classes.
* parents is an array of the representative terms for each of the equivalence classes.!

TestCase subclass: #ArrTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

!ArrTest methodsFor: 'testing - converting' stamp: 'fbs 12/28/2011 12:38'!
testAsArrayReturnsArray
	self assert: #(1 2 3) equals: (Arr wrapping: #(1 2 3)) asArray.! !

!ArrTest methodsFor: 'testing - accessing' stamp: 'fbs 12/28/2011 11:53'!
testAtPutReturnsChangedArray
	| arr arr2 |
	arr := PersistentArray wrapping: #(1 2 3).
	arr2 := arr at: 1 put: 4.
	self assert: 4 equals: (arr2 at: 1).! !

!ArrTest methodsFor: 'testing - updating' stamp: 'fbs 12/28/2011 12:26'!
testRerootOnArrIsaNoOp
	| arr |
	arr := Arr wrapping: #(1 2 3).
	self assert: arr == arr reroot.! !

TestCase subclass: #DiffTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

!DiffTest methodsFor: 'testing - converting' stamp: 'fbs 12/28/2011 12:43'!
testAsArrayReturnsArray
	| arr arr2 |
	arr := Arr wrapping: #(1 2 3).
	arr2 := arr at: 1 put: 4.
	self assert: #(4 2 3) equals: arr2 asArray! !

!DiffTest methodsFor: 'testing - accessing' stamp: 'fbs 12/28/2011 11:50'!
testAtPutReturnsChangedArray
	| arr arr2 arr3 |
	arr := PersistentArray wrapping: #(1 2 3).
	arr2 := arr at: 1 put: 4.
	self assert: 4 equals: (arr2 at: 1).
	
	arr3 := arr2 at: 2 put: 5.
	self assert: 5 equals: (arr3 at: 2).! !

!DiffTest methodsFor: 'testing - updating' stamp: 'fbs 12/28/2011 12:27'!
testRerootLeavesArraySeeminglyUnchanged
	| arr arr2 arr3 |
	arr := Arr wrapping: #(1 2 3).
	arr2 := arr at: 3 put: 4.
	arr3 := arr2 reroot.
	
	self assert: #(1 2 4) equals: arr3 asArray! !

!DiffTest methodsFor: 'testing - updating' stamp: 'fbs 12/28/2011 12:56'!
testRerootReturnsArr
	| arr arr2 arr3 |
	arr := Arr wrapping: #(1 2 3).
	arr2 := arr at: 3 put: 4.
	arr3 := arr2 reroot.
	
	self deny: arr3 isDiff.! !

TestCase subclass: #PersistentArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PersistentUnionFind-Tests'!

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/28/2011 11:59'!
testAtPutLeavesWrappedArrayUnchanged
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentArray wrapping: array.
	persistentArray := persistentArray at: 1 put: 4.

	self assert: #(3 2 1) equals: array.! !

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/28/2011 13:15'!
testAtPutReturnsaPersistentArray
	| array persistentArray persistentArray2 |
	array := #(3 2 1).
	persistentArray := PersistentArray wrapping: array.
	persistentArray2 := persistentArray at: 1 put: 4.
	self assert: #(4 2 1) equals: persistentArray2 asArray.! !

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/28/2011 11:58'!
testAtPutWorksOnImmediateReferences
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentArray wrapping: array.
	persistentArray := persistentArray at: 1 put: 4.
	self assert: 4 equals: (persistentArray at: 1).! !

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/28/2011 11:19'!
testAtWorksOnImmediateReferences
	| array persistentArray |
	array := #(3 2 1).
	persistentArray := PersistentArray wrapping: array.
	self assert: 3 equals: (persistentArray at: 1).
	self assert: 2 equals: (persistentArray at: 2).
	self assert: 1 equals: (persistentArray at: 3).! !

!PersistentArrayTest methodsFor: 'testing - accessing' stamp: 'fbs 12/28/2011 13:17'!
testPersistentArraysPersist
	| arr arr2 arr3 arr4 |
	arr := PersistentArray wrapping: #(1 2 3).
	arr2 := arr at: 1 put: 4.
	arr3 := arr2 at: 2 put: 5.
	arr4 := arr2 at: 3 put: 6.
	
	self assert: #(1 2 3) equals: arr asArray.
	self assert: #(4 2 3) equals: arr2 asArray.
	self assert: #(4 5 3) equals: arr3 asArray.
	self assert: #(4 2 6) equals: arr4 asArray.! !

!PersistentArrayTest methodsFor: 'testing - updating' stamp: 'fbs 12/28/2011 12:05'!
testRerootLeavesArraySeeminglyUnchanged
	| arr arr2 arr3 |
	arr := Arr wrapping: #(1 2 3).
	arr2 := arr at: 3 put: 4.
	arr3 := arr2 reroot.
	
	self assert: #(1 2 4) equals: arr3 asArray! !
